class InvalidParameter(Exception):
    """ Exception called if an invalid parameter is passed into an animation"""


class InvalidParameterType(Exception):
    """ Exception called if an invalid parameter type is passed into an animation"""
